local DEBUG = false

---------------------
-- Version
local RELEASETYPE = "Release"
local VERSION = GetAddOnMetadata("RaidWatch", "Version") or "1.0"
local REVISION = "407"

--[===[@alpha@
RELEASETYPE = "Alpha"
--@end-alpha@]===]

if REVISION:find("@") then 
	REVISION = "Dev" 
	RELEASETYPE = "Development"
	DEBUG = true
end

RWDEBUG = DEBUG
RWREVISION = REVISION
RWRELEASETPYE = RELEASETYPE
RWVERSION = VERSION

---------------------
-- Create Loader
RWL = LibStub("AceAddon-3.0"):NewAddon("RaidWatchLoD", "AceEvent-3.0", "AceConsole-3.0")
local RWL = RWL

---------------------
-- Locals
local L = LibStub("AceLocale-3.0"):GetLocale("RaidWatch2_Loader")
local loadOnZone = {}
local loadOnCore = {}

local monitorZones = {}
local modInfo = {}
local bossMods = {}

local BabbleZone

---------------------
-- Init
function RWL:OnInitialize()
	local defaults = {
		profile = {
			minimap = {
				hide = false,
			},
			vicotorySound = "RW: Victory",
			GlobalBossHealthFrames = false,
		},
		global = {
			ver = 1.26
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("RaidWatch2DB", defaults, "Default")
	if RW then RW.db = self.db end

	if not BabbleZone then
		BabbleZone = LibStub("LibBabble-Zone-3.0"):GetUnstrictLookupTable()
	end
	
	for i = 1, GetNumAddOns() do
		if GetAddOnMetadata(i, "X-RW-Name") then
			local modId = GetAddOnInfo(i)
			local name 	= GetAddOnMetadata(i, "X-RW-Name") or ""
			local icon = GetAddOnMetadata(i, "X-RW-Icon")
			
			if GetAddOnMetadata(i, "X-RW-LoadWithZone") then
				local zones = {strsplit(",", GetAddOnMetadata(i, "X-RW-LoadWithZone") or "")}
				for i, zone in pairs(zones) do
					--@debug@
					if not BabbleZone[zone] then
						print("Could not find in Babble-Zone", zone)
					end
					--@end-debug
					zone = BabbleZone[zone] or zone
					loadOnZone[zone] = loadOnZone[zone] or {}
					loadOnZone[zone][modId] = {
						name = name,
						icon = icon,
					}
					monitorZones[zone] = true
				end
			end
			
			if GetAddOnMetadata(i, "X-RW-BossMod") then
				bossMods[name] = true
			end
			
			if GetAddOnMetadata(i, "X-RW-LoadWithCore") then
				loadOnCore[modId] = {
					name = name,
					icon = icon,
				}
			end
			
			modInfo[name] = {
				modID = modId,
				icon = icon,
			}
		end
	end
end

---------------------
-- Enable
function RWL:OnEnable()
	if RELEASETYPE == "Alpha" then
		self:Msg(L.ALPHA_INFO)
	elseif RELEASETYPE == "Development" then
		self:Msg(L.DEV_INFO)
	end
	
	self:RegisterChatCommand("mrw", "SlashHandler")
	self:RegisterChatCommand("rw2", "SlashHandler")
	self:RegisterChatCommand("range", "ShowRange")
	self:RegisterChatCommand("rwtimerg", "ShowGlobalTimer")
	self:RegisterChatCommand("rwtimer", "ShowTimer")
	
	self:SetUpLDB()	
	
	if RWDEBUG then	
		self:LoadCore()
	else
		self:RegisterMessage("RW_Enabled")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChange")
		self:ZoneChange()
	end
end

function RWL:Msg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(("|cff28579d[|r|cffcfe846%s|r|cff28579d]|r "):format(tostring(L.LOADER_NAME))..msg, 0.39, 0.67, 0.87)
end

-------------------------------------------
-- Events
function RWL:ZoneChange()
	local zone1, zone2 = GetRealZoneText(), GetZoneText()
	if loadOnZone[zone1] then self:LoadCore(zone1) end
	if loadOnZone[zone2] then self:LoadCore(zone2) end
	
	if monitorZones[zone1] or monitorZones[zone2] then
		self:SendMessage("RW_MonitorZone")
	else
		self:SendMessage("RW_NoMonitorZone")
	end
end

-------------------------------------------
-- Messages

function RWL:RW_Enabled()
	for name, data in pairs(loadOnCore) do
		self:LoadAddOn(name)
		loadOnCore[name] = nil
	end
end

-------------------------------------------
-- Load functions
function RWL:LoadAddOn(addon)
	local loaded, errorMsg = LoadAddOn(addon)
	if not loaded then
		local name = select(2, GetAddOnInfo(addon))
		if not name then
			print("Error while loading "..addon..", it does not exist")
		else
			self:Msg(L.MOD_LOAD_ERROR:format(name, _G["ADDON_"..errorMsg]))
		end
		return
	end
	return true
end

function RWL:LoadCore(zone)
	if not RW then 
		self:LoadAddOn("RaidWatch_Core")
		if not RW then
			return 
		end
	end
	if not RW:IsEnabled() then
		RW:Enable()
		self.ldbojb.icon = [[Interface\Icons\Achievement_Boss_Sindragosa]]
	end
	self:LoadZone(zone)
end

function RWL:LoadZone(zone)
	if loadOnZone[zone] then
		for name, data in pairs(loadOnZone[zone]) do
			if RWL:LoadAddOn(name) and bossMods[data.name] then
				self:Msg("Loaded "..data.name)
				self:SendMessage("RW_BossModLoaded", data.name)
			end
			loadOnZone[zone][name] = nil
			bossMods[data.name] = nil
		end
	end
end

function RWL:LoadByName(name)
	if modInfo[name] then
		if self:LoadAddOn(modInfo[name].modID) and bossMods[name] then
			self:Msg("Loaded "..name)
			self:SendMessage("RW_BossModLoaded", name)
		end
		for zone, data in pairs(loadOnZone) do
			for mname, zonedata in pairs(data) do
				if mname == name then
					data[mname] = nil
				end
			end
		end
		bossMods[name] = nil
	end
end

function RWL:GetModInfo(name)
	return modInfo[name]
end

function RWL:IterateBossMods()
	return pairs(bossMods)
end

--------------------------------------------
-- Slash commands
function RWL:SlashHandler(input, button, ...)
	if input == "" or input == "config" then
		self:LoadCore()
		if RW then
			if not button or type(button) == "table" or button == "RightButton" then
				if not IsAddOnLoaded("RaidWatch_Options") and not RW:GetPlugin("Options") then
					if not self:LoadAddOn("RaidWatch_Options") then
						return
					end
				end
				local options = RW:GetPlugin("Options")
				if options then 
					options:Enable()
					options:Toggle() 
				end
			end
		end
	end
end

function RWL:ShowRange(input)
	local number = tonumber(input)
	if type(number) == "number" then
		self:SendMessage("RW_ShowRangeCheck", true, number)
	else
		self:SendMessage("RW_ShowRangeCheck", true, 11)
	end
end

function RWL:ShowTimer(message)
	local time, text, id = string.split("*", message)
	if not (time and text) then
		self:Msg("Usage: /rwtimerg <time mm:ss> * <text> * [id]")
		return
	end
	local min, sec = string.split(":", time)
	if not sec and min then
		sec = min
		min = 0
	end
	time = (tonumber(min or "") or 0) * 60 + (tonumber(sec) or 0)
	self:SendMessage("RW_StartBar", id or math.random(), time, text)
end

function RWL:ShowGlobalTimer(message)
	local time, text, id = string.split("*", message)
	if not (time and text) then
		self:Msg("Usage: /rwtimerg <time mm:ss> * <text> * [id]")
		return
	end
	local min, sec = string.split(":", time)
	if not sec and min then
		sec = min
		min = 0
	end
	time = (tonumber(min or "") or 0) * 60 + (tonumber(sec) or 0)
	
	self:SendMessage("RW_SendComm", "StartCommBar", nil, nil, id or math.random(), time, text)
end

---------------------------------------------
-- LDB
function RWL:SetUpLDB()
	local LDB = LibStub("LibDataBroker-1.1")
	if not LDB then return end
	local l = LDB:NewDataObject("RW2")
	l.type = "launcher"
	l.icon = [[Interface\Icons\Achievement_Boss_Sapphiron_01]]
	l.tocname = "RaidWatch_Core"
	l.label = "Raid Watch 2"
	l.OnClick = function(self, button)
		RWL:SlashHandler("config", button)
	end
	l.OnTooltipShow = function(tt)
		tt:AddLine(L.LDB_INFO)
		tt:AddLine(L.LDB_LEFTCLICK)
		tt:AddLine(L.LDB_RIGHTCLICK)
	end
	local LibDBIcon = LibStub("LibDBIcon-1.0")
	if not LibDBIcon then return end
	LibDBIcon:Register("RW2", l, self.db.profile.minimap)
	
	self.ldbojb = l
end